/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.plugin.layoutComponents;

import java.util.ArrayList;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Spinner;

import eu.DECIDEh2020.architect.appManager.models.AppDescriptor;
import eu.DECIDEh2020.architect.appManager.models.Microservice;


public class MicroserviceListComposite extends Composite {
	private AppDescriptor project;
    private ListViewer listViewerMscvs;
    private java.util.List<PageCompleteListener> pageCompleteListeners = new ArrayList<PageCompleteListener>();
    private java.util.List<ContentChangeListener> contentChangeListeners = new ArrayList<ContentChangeListener>();
    
	private static class MicroserviceLabelProvider extends LabelProvider {
		public Image getImage(Object element) {
			return null;
		}
		public String getText(Object element) {
			return  ((Microservice) element).getName();
		}
	}
	
	
	private static class MicroserviceContentProvider implements IStructuredContentProvider {
		public Object[] getElements(Object inputElement) {
			return ((AppDescriptor) inputElement).getMicroservices().toArray();
		}
		public void dispose() {
		}
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}
    
    
    
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public MicroserviceListComposite(Composite parent, int style, AppDescriptor project, Boolean showListLabel, Color background) {
		super(parent, SWT.NONE);
		this.project = project;
		setBackground(background);
		setLayout(new GridLayout(2, false));
        
        Composite composite = new Composite(this, SWT.NONE);
        composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
        composite.setLayout(new GridLayout(2, false));
        composite.setBackground(background);
		
				  
		      // only show this label if needed
		        if(showListLabel) {
		        Label lblListMscv = new Label(composite, SWT.NONE);
		        lblListMscv.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
		        lblListMscv.setText("List of Microservices:");
		        lblListMscv.setBackground(background);
		        }
		
		listViewerMscvs = new ListViewer(composite, SWT.BORDER | SWT.V_SCROLL);
		List listMscvs = listViewerMscvs.getList();
		listMscvs.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		listViewerMscvs.setLabelProvider(new MicroserviceLabelProvider());
		listViewerMscvs.setContentProvider(new MicroserviceContentProvider());
		listViewerMscvs.setInput(project);
		
		
		
		
		
		
		Composite compositeMscvUpDown = new Composite(this, SWT.NONE);
		compositeMscvUpDown.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1));
		FillLayout fl_compositeMscvUpDown = new FillLayout(SWT.VERTICAL);
		fl_compositeMscvUpDown.marginHeight = 5;
		fl_compositeMscvUpDown.spacing = 5;
		compositeMscvUpDown.setLayout(fl_compositeMscvUpDown);
		compositeMscvUpDown.setBackground(background);
		
		Button btnUp = new Button(compositeMscvUpDown, SWT.NONE);
		btnUp.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				project.changeIndex(listMscvs.getSelectionIndex(), -1);
				listViewerMscvs.refresh();
				setContentChanged();
			}
		});
		btnUp.setEnabled(false);
		btnUp.setText("Up");
		
		Button btnDown = new Button(compositeMscvUpDown, SWT.NONE);
		btnDown.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				//move(-1);
				project.changeIndex(listMscvs.getSelectionIndex(), 1);
				listViewerMscvs.refresh();
				setContentChanged();
			}
		});
		btnDown.setEnabled(false);
		btnDown.setText("Down");
        
        Composite composite_1 = new Composite(this, SWT.NONE);
        composite_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
        composite_1.setLayout(new GridLayout(2, false));
        composite_1.setBackground(background);
        
        Label lblAddMicroservices = new Label(composite_1, SWT.NONE);
        lblAddMicroservices.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
        lblAddMicroservices.setText("Add Microservices:");
        lblAddMicroservices.setBackground(background);
        
        Composite compositeMscvAdd = new Composite(composite_1, SWT.NONE);
        FillLayout fl_compositeMscvAdd = new FillLayout(SWT.HORIZONTAL);
        fl_compositeMscvAdd.spacing = 15;
        compositeMscvAdd.setLayout(fl_compositeMscvAdd);
        compositeMscvAdd.setBackground(background);
        
        Spinner spinner = new Spinner(compositeMscvAdd, SWT.BORDER);
        spinner.setMinimum(1);
        spinner.setSelection(1);
        
        Button btnAddMscv = new Button(compositeMscvAdd, SWT.NONE);
        btnAddMscv.addSelectionListener(new SelectionAdapter() {
        	@Override
        	public void widgetSelected(SelectionEvent e) {
        		
        		project.addMicroservices(spinner.getSelection());
        		listViewerMscvs.refresh();
        		setContentChanged();
        		setPageComplete(project.getMicroservices().size()>0);
        	}
        });
        btnAddMscv.setText("Add");
       
        
        Button btnRemoveMsvc = new Button(this, SWT.NONE);
        btnRemoveMsvc.addSelectionListener(new SelectionAdapter() {
        	@Override
        	public void widgetSelected(SelectionEvent e) {
        		project.removeMicroservice(listMscvs.getSelectionIndex());
        		 listViewerMscvs.refresh();
        		 setContentChanged();
        		 setPageComplete(project.getMicroservices().size()>0);
        	}
        });
        btnRemoveMsvc.setEnabled(false);
        btnRemoveMsvc.setText("Remove");
		
        
        listMscvs.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				
				if(listMscvs.getSelectionCount()==1) {
					//TODO: Disabling buttons if topmost or bottommost does not work
					if(listMscvs.getSelectionIndex()!=0)btnUp.setEnabled(true);
					if(listMscvs.getSelectionIndex()!=listMscvs.getItemCount()-1)btnDown.setEnabled(true);
					btnRemoveMsvc.setEnabled(true);
				}else if(listMscvs.getSelectionCount()>0) {
					btnRemoveMsvc.setEnabled(true);
					btnUp.setEnabled(false);
					btnDown.setEnabled(false);
					
				}else {
					btnRemoveMsvc.setEnabled(false);
					btnUp.setEnabled(false);
					btnDown.setEnabled(false);
				}
			}
		});
		
	}

	public void refresh() {
		listViewerMscvs.refresh();
	}
	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
	
	public void addContentChangeListener(ContentChangeListener contentChangedListener) {
		contentChangeListeners.add(contentChangedListener);
    }
	
	public void addPageCompleteListener(PageCompleteListener pageCompleteListener) {
        pageCompleteListeners.add(pageCompleteListener);
    }
	
	public void setContentChanged() {
		for(ContentChangeListener ccl : contentChangeListeners) {
			ccl.contentChanged();
		}
	}
	
	public void setPageComplete(Boolean complete) {
		for(PageCompleteListener pcl : pageCompleteListeners) {
			pcl.pageCompleted(complete);
		}
	}

}
