/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.plugin.layoutComponents;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.List;


import eu.DECIDEh2020.architect.cloudpatterns.Patterns;

import java.util.Arrays;
import java.util.EnumSet;
import java.util.HashSet;

import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.Statement;
import org.apache.jena.vocabulary.DCTerms;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;

public class NfrListViewer extends Composite {
	private static class NFRLabelProvider extends LabelProvider {
		public Image getImage(Object element) {
			return null;
		}
		public String getText(Object element) {
			return  ((Resource) element).getProperty(DCTerms.title).getString();
		}
	}
	
	
	private static class NFRContentProvider implements IStructuredContentProvider {
		@SuppressWarnings("unchecked")
		public Object[] getElements(Object inputElement) {
			return ((java.util.List<Resource>) inputElement).toArray();
			
		}
		public void dispose() {
		}
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}
	
	private List list;
	private ListViewer listViewer;
	
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public NfrListViewer(Composite parent, int style) {
		super(parent, SWT.NONE);
		setLayout(new FillLayout(SWT.HORIZONTAL));
		Patterns patterns = Patterns.createDefaultPatterns();
		java.util.List<Resource> nfrs = patterns.listNFRs();
		
		
		listViewer = new ListViewer(this, style);
		list = listViewer.getList();
		
		
		listViewer.setLabelProvider(new NFRLabelProvider());
		listViewer.setContentProvider(new NFRContentProvider());
		listViewer.setInput(nfrs);
		

	}

	
	public java.util.List<Resource> getSelection(){
		@SuppressWarnings("unchecked")
		java.util.List<Resource> selection = listViewer.getStructuredSelection().toList();
		
		return selection;
		
		
		
		
	}
	
	
	public void addSelection(java.util.List<Resource> nfrs) {
		
		
		HashSet<String> selection = new HashSet<String>(Arrays.asList(list.getSelection()));
		
		for(Resource nfr: nfrs) {
			Statement title = nfr.getProperty(DCTerms.title);
			String stitle = title.getString();
			selection.add(stitle);
		}
		list.setSelection(selection.toArray(new String[selection.size()]));
		
	}
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
	
	public void addSelectionListener(SelectionAdapter a) {
		list.addSelectionListener(a);
	}
	
	
}
