/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */

/**
 * 
 */
package eu.DECIDEh2020.architect.appManager.persistence;

import java.lang.reflect.Type;

import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.ResourceFactory;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.InstanceCreator;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonParser;
import com.google.gson.JsonPrimitive;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import com.google.gson.JsonSyntaxException;

import eu.DECIDEh2020.architect.appManager.models.AppDescriptor;
import eu.DECIDEh2020.architect.cloudpatterns.Patterns;
/**
 * @author tja
 *
 */
public class JsonHandler {

	public static class AppDescriptorWrapper {
		// Old version means an older version than the one this class currently uses
		private boolean oldVersion = false;
		private String fileVersion = "";
		private AppDescriptor appDescriptor = null;

		public AppDescriptor getAppDescriptor() {
			return appDescriptor;
		}

		public void setAppDescriptor(AppDescriptor appDescriptor) {
			this.appDescriptor = appDescriptor;
		}

		public boolean isOldVersion() {
			return oldVersion;
		}

		public void setOldVersion(boolean oldVersion) {
			this.oldVersion = oldVersion;
		}

		public String getFileVersion() {
			return fileVersion;
		}

		public void setFileVersion(String fileVersion) {
			this.fileVersion = fileVersion;
		}
	}

	private static class ResourceSerializer implements JsonSerializer<Resource> {
		public JsonElement serialize(Resource src, Type typeOfSrc, JsonSerializationContext context) {
			return new JsonPrimitive(src.toString());
		}
	}

	private static class ResourceDeserializer implements JsonDeserializer<Resource> {
		public Resource deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context)
				throws JsonParseException {

			return Patterns.createDefaultPatterns().getModel().createResource(json.getAsJsonPrimitive().getAsString());
		}
	}

	private static class ResourceInstanceCreator implements InstanceCreator<Resource> {
		public Resource createInstance(Type type) {

			return ResourceFactory.createResource();
		}
	}

	/**
	 * @param jsonString the String in JSON format to convert into an AppDescriptor Object
	 * @return AppDescriptorWrapper a wrapper Object containing the AppDescriptor and a Boolean value which might provide the reason for possible conversion errors by indicating the version of the AppDescriptor file. 
	 * @throws JsonSyntaxException TODO
	 */
	public static AppDescriptorWrapper toAppDescriptor(String jsonString) throws JsonSyntaxException {
		AppDescriptorWrapper app = new AppDescriptorWrapper();

		Gson gson = new GsonBuilder().registerTypeAdapter(Resource.class, new ResourceInstanceCreator())
				.registerTypeAdapter(Resource.class, new ResourceDeserializer()).create();

		app.appDescriptor = gson.fromJson(jsonString, AppDescriptor.class);

		JsonParser parser = new JsonParser();
		JsonObject jsonObject = parser.parse(jsonString).getAsJsonObject();

		if (jsonObject.has("version")) {
			String version = parser.parse(jsonString).getAsJsonObject().get("version").getAsString();

			if (!version.equals(AppDescriptor.plugin_version)) {
				app.setOldVersion(true);
				app.setFileVersion(version);
			}

		}
		return app;

	}
	
	public static String fromAppDescriptor(AppDescriptor app) {
		//create new gson with pretty printing activated and Serializer & deserializer for Resource Type
		Gson gson = new GsonBuilder()
				.setPrettyPrinting()
				.registerTypeAdapter(Resource.class, new ResourceSerializer())
				.registerTypeAdapter(Resource.class, new ResourceDeserializer())
				.create();
		
		
		
		//convert app descriptor object to json string
		return gson.toJson(app);
		
	}
	

}
